@@ -52,14 +52,14 @@ struct notcurses* notcurses_init(const notcurses_options*, FILE*);
 void notcurses_version_components(int* major, int* minor, int* patch, int* tweak);
 int notcurses_lex_margins(const char* op, notcurses_options* opts);
 int notcurses_stop(struct notcurses*);
-int notcurses_render(struct notcurses*);
+int notcurses_render(struct notcurses* nc);
+int notcurses_render_to_buffer(struct notcurses* nc, char** buf, size_t* buflen);
 int notcurses_render_to_file(struct notcurses* nc, FILE* fp);
 struct ncplane* notcurses_stdplane(struct notcurses*);
 const struct ncplane* notcurses_stdplane_const(const struct notcurses* nc);
 void ncplane_set_channels(struct ncplane* n, uint64_t channels);
 uint64_t ncplane_set_fchannel(struct ncplane* n, uint32_t channel);
 uint64_t ncplane_set_bchannel(struct ncplane* n, uint32_t channel);
-void ncplane_set_attr(struct ncplane* n, unsigned stylebits);
 int ncplane_set_base_cell(struct ncplane* ncp, const cell* c);
 int ncplane_set_base(struct ncplane* ncp, const char* egc, uint32_t styles, uint64_t channels);
 int ncplane_base(struct ncplane* ncp, cell* c);
@@ -120,14 +120,14 @@ char* notcurses_at_yx(struct notcurses* nc, int yoff, int xoff, uint16_t* stylem
 char* ncplane_at_cursor(struct ncplane* n, uint16_t* stylemask, uint64_t* channels);
 char* ncplane_at_yx(const struct ncplane* n, int y, int x, uint16_t* stylemask, uint64_t* channels);
 typedef enum {
-  NCBLIT_1x1,     // full block                █
-  NCBLIT_2x1,     // full/(upper|left) blocks  ▄█
-  NCBLIT_1x1x4,   // shaded full blocks        ▓▒░█
-  NCBLIT_2x2,     // quadrants                 ▗▐ ▖▄▟▌▙█
-  NCBLIT_4x1,     // four vert/horz levels     █▆▄▂ / ▎▌▊█
-  NCBLIT_BRAILLE, // 4 rows, 2 cols (braille)  ⡀⡄⡆⡇⢀⣀⣄⣆⣇⢠⣠⣤⣦⣧⢰⣰⣴⣶⣷⢸⣸⣼⣾⣿
-  NCBLIT_8x1,     // eight vert/horz levels    █▇▆▅▄▃▂▁ / ▏▎▍▌▋▊▉█
-  NCBLIT_SIXEL,   // 6 rows, 1 col (RGB)
+  NCBLIT_1x1,
+  NCBLIT_2x1,
+  NCBLIT_2x2,
+  NCBLIT_3x2,
+  NCBLIT_4x1,
+  NCBLIT_BRAILLE,
+  NCBLIT_8x1,
+  NCBLIT_SIXEL,
 } ncblitter_e;
 const char* notcurses_str_blitter(ncblitter_e blitter);
 int notcurses_lex_blitter(const char* op, ncblitter_e* blitter);
@@ -135,11 +135,8 @@ uint32_t* ncplane_rgba(const struct ncplane* nc, ncblitter_e blit, int begy, int
 char* ncplane_contents(const struct ncplane* nc, int begy, int begx, int leny, int lenx);
 void* ncplane_set_userptr(struct ncplane* n, void* opaque);
 void* ncplane_userptr(struct ncplane* n);
-int ncplane_resize(struct ncplane* n, int keepy, int keepx, int keepleny,
-                       int keeplenx, int yoff, int xoff, int ylen, int xlen);
+int ncplane_resize(struct ncplane* n, int keepy, int keepx, int keepleny, int keeplenx, int yoff, int xoff, int ylen, int xlen);
 uint64_t ncplane_channels(const struct ncplane* n);
-uint16_t ncplane_attr(const struct ncplane* n);
-unsigned ncplane_styles(const struct ncplane* n);
 int ncplane_set_fg_rgb8(struct ncplane* n, int r, int g, int b);
 int ncplane_set_bg_rgb8(struct ncplane* n, int r, int g, int b);
 void ncplane_set_fg_rgb8_clipped(struct ncplane* n, int r, int g, int b);
@@ -152,9 +149,10 @@ int ncplane_set_fg_alpha(struct ncplane* n, unsigned alpha);
 int ncplane_set_bg_alpha(struct ncplane* n, unsigned alpha);
 int ncplane_set_fg_palindex(struct ncplane* n, int idx);
 int ncplane_set_bg_palindex(struct ncplane* n, int idx);
-void ncplane_styles_set(struct ncplane* n, unsigned stylebits);
-void ncplane_styles_on(struct ncplane* n, unsigned stylebits);
-void ncplane_styles_off(struct ncplane* n, unsigned stylebits);
+unsigned ncplane_styles(const struct ncplane* n);
+void ncplane_set_styles(struct ncplane* n, unsigned stylebits);
+void ncplane_on_styles(struct ncplane* n, unsigned stylebits);
+void ncplane_off_styles(struct ncplane* n, unsigned stylebits);
 typedef struct ncstats {
   uint64_t renders;          // number of successful notcurses_render() runs
   uint64_t failed_renders;   // number of aborted renders, should be 0
@@ -174,8 +172,9 @@ typedef struct ncstats {
   uint64_t defaultelisions;  // default color was emitted
   uint64_t defaultemissions; // default color was elided
 } ncstats;
+ncstats* notcurses_stats_alloc(struct notcurses* nc);
 void notcurses_stats(struct notcurses* nc, ncstats* stats);
-void notcurses_reset_stats(struct notcurses* nc, ncstats* stats);
+void notcurses_stats_reset(struct notcurses* nc, ncstats* stats);
 int ncplane_hline_interp(struct ncplane* n, const cell* c, int len, uint64_t c1, uint64_t c2);
 int ncplane_vline_interp(struct ncplane* n, const cell* c, int len, uint64_t c1, uint64_t c2);
 int ncplane_box(struct ncplane* n, const cell* ul, const cell* ur, const cell* ll, const cell* lr, const cell* hline, const cell* vline, int ystop, int xstop, unsigned ctlword);
@@ -213,6 +212,7 @@ struct ncvisual* ncvisual_from_plane(const struct ncplane* n, ncblitter_e blit,
 int ncvisual_geom(const struct notcurses* nc, const struct ncvisual* n, const struct ncvisual_options* vopts, int* y, int* x, int* toy, int* tox);
 void ncvisual_destroy(struct ncvisual* ncv);
 int ncvisual_decode(struct ncvisual* nc);
+int ncvisual_decode_loop(struct ncvisual* nc);
 int ncvisual_rotate(struct ncvisual* n, double rads);
 int ncvisual_resize(struct ncvisual* n, int rows, int cols);
 int ncvisual_polyfill_yx(struct ncvisual* n, int y, int x, uint32_t rgba);
@@ -315,6 +315,7 @@ int ncmenu_nextsection(struct ncmenu* n);
 int ncmenu_prevsection(struct ncmenu* n);
 int ncmenu_nextitem(struct ncmenu* n);
 int ncmenu_previtem(struct ncmenu* n);
+int ncmenu_item_set_status(struct ncmenu* n, const char* section, const char* item, bool enabled);
 const char* ncmenu_selected(const struct ncmenu* n, struct ncinput* ni);
 bool ncmenu_offer_input(struct ncmenu* n, const struct ncinput* nc);
 int ncmenu_destroy(struct ncmenu* n);
@@ -337,9 +338,9 @@ int ncreel_redraw(struct ncreel* pr);
 struct nctablet* ncreel_focused(struct ncreel* pr);
 struct nctablet* ncreel_next(struct ncreel* pr);
 struct nctablet* ncreel_prev(struct ncreel* pr);
-int ncreel_destroy(struct ncreel* pr);
+void ncreel_destroy(struct ncreel* pr);
 void* nctablet_userptr(struct nctablet* t);
-struct ncplane* nctablet_ncplane(struct nctablet* t);
+struct ncplane* nctablet_plane(struct nctablet* t);
 int ncplane_polyfill_yx(struct ncplane* n, int y, int x, const cell* c);
 int ncplane_gradient(struct ncplane* n, const char* egc, uint32_t styles, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ystop, int xstop);
 int ncplane_highgradient(struct ncplane* n, uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr, int ystop, int xstop);
@@ -416,6 +417,7 @@ int ncdirect_bg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
 int ncdirect_fg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
 unsigned ncdirect_palette_size(const struct ncdirect* nc);
 int ncdirect_putstr(struct ncdirect* nc, uint64_t channels, const char* utf8);
+int ncdirect_printf_aligned(struct ncdirect* n, int y, ncalign_e align, const char* fmt, ...);
 int ncdirect_fg_rgb(struct ncdirect* n, unsigned rgb);
 int ncdirect_bg_rgb(struct ncdirect* n, unsigned rgb);
 int ncdirect_styles_set(struct ncdirect* n, unsigned stylebits);
