@@ -1,28 +1,30 @@
 // functions already exported by bindgen : 0
 // ------------------------------------------
 //
-// static inline functions to reimplement: 2
-// ------------------------------------------ (done / (x) wont / remaining)
-// (+) implement : 2 / 0 / 0
-// (#) unit tests: 0 / 0 / 2
+// static inline functions total: 2
+// ------------------------------------------ (done / remaining)
+// (+) done: 2 / 0
+// (#) test: 0 / 2
 // ------------------------------------------
-//+nckey_mouse_p
-//+nckey_supppuab_p
+//+ nckey_mouse_p
+//+ nckey_supppuab_p
 
-use crate as nc;
+use crate::{NCKEY_BUTTON1, NCKEY_RELEASE};
 
-/// Is this char32_t a Supplementary Private Use Area-B codepoint?
-// TODO: TEST
+/// Is this u32 a Supplementary Private Use Area-B codepoint?
+///
+/// Links:
+/// - https://en.wikipedia.org/wiki/Private_Use_Areas
+/// - https://codepoints.net/supplementary_private_use_area-b
 #[inline]
 pub fn nckey_supppuab_p(w: u32) -> bool {
     w >= 0x100000 && w <= 0x10fffd
 }
 
 /// Is the event a synthesized mouse event?
-// TODO: TEST
 #[inline]
 pub fn nckey_mouse_p(r: u32) -> bool {
-    r >= nc::NCKEY_BUTTON1 && r <= nc::NCKEY_RELEASE
+    r >= NCKEY_BUTTON1 && r <= NCKEY_RELEASE
 }
 
 #[cfg(test)]
