@@ -1,8 +1,7 @@
-// functions already exported by bindgen : 94
+// functions already exported by bindgen : 97
 // ------------------------------------------
 // ncplane_above
 // ncplane_at_cursor
-// ncplane_attr
 // ncplane_at_yx
 // ncplane_base
 // ncplane_below
@@ -35,8 +34,11 @@
 // ncplane_move_bottom
 // ncplane_move_top
 // ncplane_move_yx
+// ncplane_new
 // ncplane_notcurses
 // ncplane_notcurses_const
+// ncplane_off_styles
+// ncplane_on_styles
 // ncplane_parent
 // ncplane_parent_const
 // ncplane_polyfill_yx
@@ -60,7 +62,6 @@
 // ncplane_rgba
 // ncplane_rotate_ccw
 // ncplane_rotate_cw
-// ncplane_set_attr
 // ncplane_set_base
 // ncplane_set_base_cell
 // ncplane_set_bchannel
@@ -79,8 +80,10 @@
 // ncplane_set_fg_rgb8
 // ncplane_set_fg_rgb8_clipped
 // ncplane_set_scrolling
+// ncplane_set_styles
 // ncplane_set_userptr
 // ncplane_stain
+// ncplane_styles
 // ncplane_styles_off
 // ncplane_styles_on
 // ncplane_styles_set
@@ -95,57 +98,54 @@
 // ncplane_y
 // ncplane_yx
 //
-// NOTE: functions removed:
-// ncplane_styles (duplicates ncplane_attr)
-//
-// static inline functions to reimplement: 41
-// ------------------------------------------ (done / (x) wont / remaining)
-// (+) implement: 34 / 7 /  0
-// (#) unit test:  0 / 7 / 34
+// static inline functions total: 42
+// ------------------------------------------ (implement / remaining)
+// (X) wont:  8
+// (+) done: 32 / 2
+// (#) test:  0 / 34
 // ------------------------------------------
-//+ncplane_align
-//+ncplane_at_cursor_cell
-//+ncplane_at_yx_cell
-//+ncplane_bchannel
-//+ncplane_bg_alpha
-//+ncplane_bg_default_p
-//+ncplane_bg_rgb
-//+ncplane_bg_rgb8
-//+ncplane_box_sized
-//+ncplane_dim_x
-//+ncplane_dim_y
-//+ncplane_double_box
-//+ncplane_double_box_sized
-//+ncplane_fchannel
-//+ncplane_fg <<<<
-//+ncplane_fg_alpha
-//+ncplane_fg_default_p
-//+ncplane_fg_rgb
-//+ncplane_fg_rgb8
-//+ncplane_gradient_sized     // u64|u32 https://github.com/dankamongmen/notcurses/issues/920
-//+ncplane_hline
-//+ncplane_perimeter
-//+ncplane_perimeter_double
-//+ncplane_perimeter_rounded
-//+ncplane_putc
-// ncplane_putchar
-// ncplane_putchar_yx
-//+ncplane_putegc
-//+ncplane_putnstr
-//+ncplane_putstr
-//xncplane_putwc                // I don't think these will be needed from Rust. See:
-//xncplane_putwc_stained
-//xncplane_putwc_yx             // https://locka99.gitbooks.io/a-guide-to-porting-c-to-rust/content/features_of_rust/strings.html
-//xncplane_putwegc              //
-//xncplane_putwegc_yx           //
-//xncplane_putwstr              //
-//xncplane_putwstr_aligned      //
-//xncplane_putwstr_yx           //
-//+ncplane_resize_simple
-//+ncplane_rounded_box
-//+ncplane_rounded_box_sized
-//+ncplane_vline
-//+ncplane_vprintf
+//+ ncplane_align
+//+ ncplane_at_cursor_cell
+//+ ncplane_at_yx_cell
+//+ ncplane_bchannel
+//+ ncplane_bg_alpha
+//+ ncplane_bg_default_p
+//+ ncplane_bg_rgb
+//+ ncplane_bg_rgb8
+//+ ncplane_box_sized
+//+ ncplane_dim_x
+//+ ncplane_dim_y
+//+ ncplane_double_box
+//+ ncplane_double_box_sized
+//+ ncplane_fchannel
+//+ ncplane_fg_alpha
+//+ ncplane_fg_default_p
+//+ ncplane_fg_rgb
+//+ ncplane_fg_rgb8
+//+ ncplane_gradient_sized       // u64|u32 https://github.com/dankamongmen/notcurses/issues/920
+//+ ncplane_hline
+//+ ncplane_perimeter
+//+ ncplane_perimeter_double
+//+ ncplane_perimeter_rounded
+//+ ncplane_putc
+//  ncplane_putchar
+//  ncplane_putchar_yx
+//+ ncplane_putegc
+//+ ncplane_putnstr
+//+ ncplane_putstr
+//X ncplane_putwc                // I don't think these will be needed from Rust. See:
+//X ncplane_putwc_stained
+//X ncplane_putwc_yx             // https://locka99.gitbooks.io/a-guide-to-porting-c-to-rust/content/features_of_rust/strings.html
+//X ncplane_putwegc              //
+//X ncplane_putwegc_yx           //
+//X ncplane_putwstr              //
+//X ncplane_putwstr_aligned      //
+//X ncplane_putwstr_yx           //
+//+ ncplane_resize_simple
+//+ ncplane_rounded_box
+//+ ncplane_rounded_box_sized
+//+ ncplane_vline
+//+ ncplane_vprintf
 //
 // NOTE: TODO: Still remains all the ncplane_printf* functions/macros (at the end)
 
@@ -153,10 +153,21 @@ use core::ffi::c_void;
 use core::ptr::null_mut;
 
 use cstr_core::CString;
+use libc::free;
 
-use crate as nc;
-use nc::types::{
-    Align, AlphaBits, Cell, Channel, ChannelPair, Color, EGCBackstop, IntResult, Plane, StyleMask,
+use crate::{
+    bindgen::__va_list_tag,
+    cell_load, cell_release, cells_double_box, cells_rounded_box, channels_bchannel,
+    channels_bg_alpha, channels_bg_default_p, channels_bg_rgb, channels_bg_rgb8, channels_fchannel,
+    channels_fg_alpha, channels_fg_default_p, channels_fg_rgb, channels_fg_rgb8, ncplane_at_cursor,
+    ncplane_at_yx, ncplane_box, ncplane_channels, ncplane_cursor_move_yx, ncplane_cursor_yx,
+    ncplane_dim_yx, ncplane_gradient, ncplane_hline_interp, ncplane_putc_yx, ncplane_putegc_yx,
+    ncplane_putnstr_yx, ncplane_putstr_yx, ncplane_resize, ncplane_vline_interp,
+    ncplane_vprintf_yx, notcurses_align,
+    types::{
+        AlphaBits, Cell, Channel, Channels, Color, EgcBackstop, IntResult, NcAlign, NcPlane,
+        StyleMask,
+    },
 };
 
 /// Return the column at which 'cols' columns ought start in order to be aligned
@@ -164,25 +175,23 @@ use nc::types::{
 /// Undefined behavior on negative 'cols'.
 //
 // NOTE: [leave cols as i32](https://github.com/dankamongmen/notcurses/issues/904)
-// TODO: TEST
 #[inline]
-pub fn ncplane_align(plane: &Plane, align: Align, cols: i32) -> i32 {
-    nc::notcurses_align(nc::ncplane_dim_x(plane), align, cols)
+pub fn ncplane_align(plane: &NcPlane, align: NcAlign, cols: i32) -> i32 {
+    notcurses_align(ncplane_dim_x(plane), align, cols)
 }
 
 /// Retrieve the current contents of the cell under the cursor into 'cell'.
 /// This cell is invalidated if the associated plane is destroyed.
-// TODO: TEST
 #[inline]
-pub fn nplane_at_cursor_cell(plane: &mut Plane, cell: &mut Cell) -> IntResult {
-    let mut egc = unsafe { nc::ncplane_at_cursor(plane, &mut cell.stylemask, &mut cell.channels) };
+pub fn nplane_at_cursor_cell(plane: &mut NcPlane, cell: &mut Cell) -> IntResult {
+    let mut egc = unsafe { ncplane_at_cursor(plane, &mut cell.stylemask, &mut cell.channels) };
     if egc.is_null() {
         return -1;
     }
-    let result: IntResult = unsafe { nc::cell_load(plane, cell, egc) };
+    let result: IntResult = unsafe { cell_load(plane, cell, egc) };
     if result < 0 {
         unsafe {
-            nc::free(&mut egc as *mut _ as *mut c_void);
+            free(&mut egc as *mut _ as *mut c_void);
         }
     }
     result
@@ -190,19 +199,17 @@ pub fn nplane_at_cursor_cell(plane: &mut Plane, cell: &mut Cell) -> IntResult {
 
 /// Retrieve the current contents of the specified cell into 'cell'.
 /// This cell is invalidated if the associated plane is destroyed.
-// TODO: TEST
 #[inline]
-pub fn ncplane_at_yx_cell(plane: &mut Plane, y: i32, x: i32, cell: &mut Cell) -> IntResult {
-    let mut egc =
-        unsafe { nc::ncplane_at_yx(plane, y, x, &mut cell.stylemask, &mut cell.channels) };
+pub fn ncplane_at_yx_cell(plane: &mut NcPlane, y: i32, x: i32, cell: &mut Cell) -> IntResult {
+    let mut egc = unsafe { ncplane_at_yx(plane, y, x, &mut cell.stylemask, &mut cell.channels) };
     if egc.is_null() {
         return -1;
     }
     let channels = cell.channels; // need to preserve wide flag
-    let result: IntResult = unsafe { nc::cell_load(plane, cell, egc) };
+    let result: IntResult = unsafe { cell_load(plane, cell, egc) };
     cell.channels = channels;
     unsafe {
-        nc::free(&mut egc as *mut _ as *mut c_void);
+        free(&mut egc as *mut _ as *mut c_void);
     }
     result
 }
@@ -210,10 +217,9 @@ pub fn ncplane_at_yx_cell(plane: &mut Plane, y: i32, x: i32, cell: &mut Cell) ->
 /// Draw a box with its upper-left corner at the current cursor position, having
 /// dimensions 'ylen'x'xlen'. See ncplane_box() for more information. The
 /// minimum box size is 2x2, and it cannot be drawn off-screen.
-// TODO: TEST
 #[inline]
 pub fn ncplane_box_sized(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     ul: &Cell,
     ur: &Cell,
     ll: &Cell,
@@ -226,8 +232,8 @@ pub fn ncplane_box_sized(
 ) -> IntResult {
     let (mut y, mut x) = (0, 0);
     unsafe {
-        nc::ncplane_cursor_yx(plane, &mut y, &mut x);
-        nc::ncplane_box(
+        ncplane_cursor_yx(plane, &mut y, &mut x);
+        ncplane_box(
             plane,
             ul,
             ur,
@@ -243,33 +249,31 @@ pub fn ncplane_box_sized(
 }
 
 ///
-// TODO: TEST
 #[inline]
-pub fn ncplane_dim_x(plane: &Plane) -> i32 {
+pub fn ncplane_dim_x(plane: &NcPlane) -> i32 {
     unsafe {
         let mut x = 0;
-        nc::ncplane_dim_yx(plane, null_mut(), &mut x);
+        ncplane_dim_yx(plane, null_mut(), &mut x);
         x
     }
 }
 
 ///
-// TODO: TEST
 #[inline]
-pub fn ncplane_dim_y(plane: &Plane) -> i32 {
+pub fn ncplane_dim_y(plane: &NcPlane) -> i32 {
     unsafe {
         let mut y = 0;
-        nc::ncplane_dim_yx(plane, &mut y, null_mut());
+        ncplane_dim_yx(plane, &mut y, null_mut());
         y
     }
 }
 
-// TODO: TEST
+///
 #[inline]
 pub fn ncplane_double_box(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     stylemask: StyleMask,
-    channels: ChannelPair,
+    channels: Channels,
     ystop: i32,
     xstop: i32,
     ctlword: u32,
@@ -285,7 +289,7 @@ pub fn ncplane_double_box(
     let mut vl = cell_trivial_initializer![];
 
     unsafe {
-        ret = nc::cells_double_box(
+        ret = cells_double_box(
             plane,
             stylemask as u32,
             channels,
@@ -297,32 +301,32 @@ pub fn ncplane_double_box(
             &mut vl,
         );
         if ret == 0 {
-            ret = nc::ncplane_box(plane, &ul, &ur, &ll, &lr, &hl, &vl, ystop, xstop, ctlword);
+            ret = ncplane_box(plane, &ul, &ur, &ll, &lr, &hl, &vl, ystop, xstop, ctlword);
         }
 
-        nc::cell_release(plane, &mut ul);
-        nc::cell_release(plane, &mut ur);
-        nc::cell_release(plane, &mut ll);
-        nc::cell_release(plane, &mut lr);
-        nc::cell_release(plane, &mut hl);
-        nc::cell_release(plane, &mut vl);
+        cell_release(plane, &mut ul);
+        cell_release(plane, &mut ur);
+        cell_release(plane, &mut ll);
+        cell_release(plane, &mut lr);
+        cell_release(plane, &mut hl);
+        cell_release(plane, &mut vl);
     }
     ret
 }
 
-// TODO: TEST
+///
 #[inline]
 pub fn ncplane_double_box_sized(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     stylemask: StyleMask,
-    channels: ChannelPair,
+    channels: Channels,
     ylen: i32,
     xlen: i32,
     ctlword: u32,
 ) -> IntResult {
     let (mut y, mut x) = (0, 0);
     unsafe {
-        nc::ncplane_cursor_yx(plane, &mut y, &mut x);
+        ncplane_cursor_yx(plane, &mut y, &mut x);
     }
     ncplane_double_box(
         plane,
@@ -335,17 +339,15 @@ pub fn ncplane_double_box_sized(
 }
 
 /// On error, return the negative number of cells drawn.
-// TODO: TEST
 #[inline]
-pub fn ncplane_hline(plane: &mut Plane, cell: &Cell, len: i32) -> i32 {
-    unsafe { nc::ncplane_hline_interp(plane, cell, len, cell.channels, cell.channels) }
+pub fn ncplane_hline(plane: &mut NcPlane, cell: &Cell, len: i32) -> i32 {
+    unsafe { ncplane_hline_interp(plane, cell, len, cell.channels, cell.channels) }
 }
 
 ///
-// TODO: TEST
 #[inline]
 pub fn ncplane_perimeter(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     ul: &Cell,
     ur: &Cell,
     ll: &Cell,
@@ -355,27 +357,27 @@ pub fn ncplane_perimeter(
     ctlword: u32,
 ) -> IntResult {
     unsafe {
-        nc::ncplane_cursor_move_yx(plane, 0, 0);
+        ncplane_cursor_move_yx(plane, 0, 0);
         let (mut dimy, mut dimx) = (0, 0);
-        nc::ncplane_dim_yx(plane, &mut dimy, &mut dimx);
+        ncplane_dim_yx(plane, &mut dimy, &mut dimx);
         ncplane_box_sized(plane, ul, ur, ll, lr, hline, vline, dimy, dimx, ctlword)
     }
 }
 
-// TODO: TEST
+///
 #[inline]
 pub fn ncplane_perimeter_double(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     stylemask: StyleMask,
-    channels: ChannelPair,
+    channels: Channels,
     ctlword: u32,
 ) -> IntResult {
-    if unsafe { nc::ncplane_cursor_move_yx(plane, 0, 0) } != 0 {
+    if unsafe { ncplane_cursor_move_yx(plane, 0, 0) } != 0 {
         return -1;
     }
     let (mut dimy, mut dimx) = (0, 0);
     unsafe {
-        nc::ncplane_dim_yx(plane, &mut dimy, &mut dimx);
+        ncplane_dim_yx(plane, &mut dimy, &mut dimx);
     }
     let mut ul = cell_trivial_initializer![];
     let mut ur = cell_trivial_initializer![];
@@ -384,7 +386,7 @@ pub fn ncplane_perimeter_double(
     let mut hl = cell_trivial_initializer![];
     let mut vl = cell_trivial_initializer![];
     if unsafe {
-        nc::cells_double_box(
+        cells_double_box(
             plane,
             stylemask as u32,
             channels,
@@ -401,30 +403,30 @@ pub fn ncplane_perimeter_double(
     }
     let ret = ncplane_box_sized(plane, &ul, &ur, &ll, &lr, &hl, &vl, dimy, dimx, ctlword);
     unsafe {
-        nc::cell_release(plane, &mut ul);
-        nc::cell_release(plane, &mut ur);
-        nc::cell_release(plane, &mut ll);
-        nc::cell_release(plane, &mut lr);
-        nc::cell_release(plane, &mut hl);
-        nc::cell_release(plane, &mut vl);
+        cell_release(plane, &mut ul);
+        cell_release(plane, &mut ur);
+        cell_release(plane, &mut ll);
+        cell_release(plane, &mut lr);
+        cell_release(plane, &mut hl);
+        cell_release(plane, &mut vl);
     }
     ret
 }
 
-// TODO: TEST!
+///
 #[inline]
 pub fn ncplane_perimeter_rounded(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     stylemask: StyleMask,
-    channels: ChannelPair,
+    channels: Channels,
     ctlword: u32,
 ) -> IntResult {
-    if unsafe { nc::ncplane_cursor_move_yx(plane, 0, 0) } != 0 {
+    if unsafe { ncplane_cursor_move_yx(plane, 0, 0) } != 0 {
         return -1;
     }
     let (mut dimy, mut dimx) = (0, 0);
     unsafe {
-        nc::ncplane_dim_yx(plane, &mut dimy, &mut dimx);
+        ncplane_dim_yx(plane, &mut dimy, &mut dimx);
     }
     let mut ul = cell_trivial_initializer![];
     let mut ur = cell_trivial_initializer![];
@@ -433,7 +435,7 @@ pub fn ncplane_perimeter_rounded(
     let mut hl = cell_trivial_initializer![];
     let mut vl = cell_trivial_initializer![];
     if unsafe {
-        nc::cells_rounded_box(
+        cells_rounded_box(
             plane,
             stylemask as u32,
             channels,
@@ -450,36 +452,33 @@ pub fn ncplane_perimeter_rounded(
     }
     let ret = ncplane_box_sized(plane, &ul, &ur, &ll, &lr, &hl, &vl, dimy, dimx, ctlword);
     unsafe {
-        nc::cell_release(plane, &mut ul);
-        nc::cell_release(plane, &mut ur);
-        nc::cell_release(plane, &mut ll);
-        nc::cell_release(plane, &mut lr);
-        nc::cell_release(plane, &mut hl);
-        nc::cell_release(plane, &mut vl);
+        cell_release(plane, &mut ul);
+        cell_release(plane, &mut ur);
+        cell_release(plane, &mut ll);
+        cell_release(plane, &mut lr);
+        cell_release(plane, &mut hl);
+        cell_release(plane, &mut vl);
     }
     ret
 }
 
 /// Call ncplane_putc_yx() for the current cursor location.
-// TODO: TEST
 #[inline]
-pub fn ncplane_putc(plane: &mut Plane, cell: &Cell) -> IntResult {
-    unsafe { nc::ncplane_putc_yx(plane, -1, -1, cell) }
+pub fn ncplane_putc(plane: &mut NcPlane, cell: &Cell) -> IntResult {
+    unsafe { ncplane_putc_yx(plane, -1, -1, cell) }
 }
 
 /// Call ncplane_putegc() at the current cursor location.
-// TODO: TEST
 #[inline]
-pub fn ncplane_putegc(plane: &mut Plane, gcluster: i8, sbytes: &mut i32) -> IntResult {
-    unsafe { nc::ncplane_putegc_yx(plane, -1, -1, &gcluster, sbytes) }
+pub fn ncplane_putegc(plane: &mut NcPlane, gcluster: i8, sbytes: &mut i32) -> IntResult {
+    unsafe { ncplane_putegc_yx(plane, -1, -1, &gcluster, sbytes) }
 }
 
 ///
-// TODO: TEST
 #[inline]
-pub fn ncplane_putstr(plane: &mut Plane, gclustarr: &[u8]) -> IntResult {
+pub fn ncplane_putstr(plane: &mut NcPlane, gclustarr: &[u8]) -> IntResult {
     unsafe {
-        nc::ncplane_putstr_yx(
+        ncplane_putstr_yx(
             plane,
             -1,
             -1,
@@ -489,11 +488,10 @@ pub fn ncplane_putstr(plane: &mut Plane, gclustarr: &[u8]) -> IntResult {
 }
 
 ///
-// TODO: TEST
 #[inline]
-pub fn ncplane_putnstr(plane: &mut Plane, size: nc::size_t, gclustarr: &[u8]) -> IntResult {
+pub fn ncplane_putnstr(plane: &mut NcPlane, size: u64, gclustarr: &[u8]) -> IntResult {
     unsafe {
-        nc::ncplane_putnstr_yx(
+        ncplane_putnstr_yx(
             plane,
             -1,
             -1,
@@ -505,12 +503,11 @@ pub fn ncplane_putnstr(plane: &mut Plane, size: nc::size_t, gclustarr: &[u8]) ->
 
 /// Resize the plane, retaining what data we can (everything, unless we're
 /// shrinking in some dimension). Keep the origin where it is.
-// TODO: TEST
 #[inline]
-pub fn ncplane_resize_simple(plane: &mut Plane, ylen: i32, xlen: i32) -> IntResult {
+pub fn ncplane_resize_simple(plane: &mut NcPlane, ylen: i32, xlen: i32) -> IntResult {
     let (mut oldy, mut oldx) = (0, 0);
     unsafe {
-        nc::ncplane_dim_yx(plane, &mut oldy, &mut oldx);
+        ncplane_dim_yx(plane, &mut oldy, &mut oldx);
     }
     let keepleny = {
         if oldy > ylen {
@@ -526,22 +523,21 @@ pub fn ncplane_resize_simple(plane: &mut Plane, ylen: i32, xlen: i32) -> IntResu
             oldx
         }
     };
-    unsafe { nc::ncplane_resize(plane, 0, 0, keepleny, keeplenx, 0, 0, ylen, xlen) }
+    unsafe { ncplane_resize(plane, 0, 0, keepleny, keeplenx, 0, 0, ylen, xlen) }
 }
 
 ///
 /// On error, return the negative number of cells drawn.
-// TODO: TEST
 #[inline]
-pub fn ncplane_vline(plane: &mut Plane, cell: &Cell, len: i32) -> i32 {
-    unsafe { nc::ncplane_vline_interp(plane, cell, len, cell.channels, cell.channels) }
+pub fn ncplane_vline(plane: &mut NcPlane, cell: &Cell, len: i32) -> i32 {
+    unsafe { ncplane_vline_interp(plane, cell, len, cell.channels, cell.channels) }
 }
 
-// TODO: TEST
+///
 #[inline]
-pub fn ncplane_vprintf(plane: &mut Plane, format: &str, ap: &mut nc::__va_list_tag) -> IntResult {
+pub fn ncplane_vprintf(plane: &mut NcPlane, format: &str, ap: &mut __va_list_tag) -> IntResult {
     unsafe {
-        nc::ncplane_vprintf_yx(
+        ncplane_vprintf_yx(
             plane,
             -1,
             -1,
@@ -554,11 +550,10 @@ pub fn ncplane_vprintf(plane: &mut Plane, format: &str, ap: &mut nc::__va_list_t
 /// Draw a gradient with its upper-left corner at the current cursor position,
 /// having dimensions 'ylen'x'xlen'. See ncplane_gradient for more information.
 /// static inline int
-// TODO: TEST
 // XXX receive cells as u32? https://github.com/dankamongmen/notcurses/issues/920
 #[inline]
 pub fn ncplane_gradient_sized(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     egc: &[u8],
     stylemask: StyleMask,
     ul: u64,
@@ -573,8 +568,8 @@ pub fn ncplane_gradient_sized(
     }
     let (mut y, mut x) = (0, 0);
     unsafe {
-        nc::ncplane_cursor_yx(plane, &mut y, &mut x);
-        nc::ncplane_gradient(
+        ncplane_cursor_yx(plane, &mut y, &mut x);
+        ncplane_gradient(
             plane,
             CString::new(egc).expect("Bad EGC").as_ptr(),
             stylemask as u32,
@@ -589,91 +584,81 @@ pub fn ncplane_gradient_sized(
 }
 
 /// Extract the 32-bit working foreground channel from an ncplane.
-// TODO: TEST
 #[inline]
-pub fn ncplane_fchannel(plane: &Plane) -> Channel {
-    nc::channels_fchannel(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_fchannel(plane: &NcPlane) -> Channel {
+    channels_fchannel(unsafe { ncplane_channels(plane) })
 }
 
 /// Extract the 32-bit working background channel from an ncplane.
-// TODO: TEST
 #[inline]
-pub fn ncplane_bchannel(plane: &Plane) -> Channel {
-    nc::channels_bchannel(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_bchannel(plane: &NcPlane) -> Channel {
+    channels_bchannel(unsafe { ncplane_channels(plane) })
 }
 
 /// Extract 24 bits of working foreground RGB from an ncplane, shifted to LSBs.
-// TODO: TEST
 #[inline]
-pub fn ncplane_fg_rgb(plane: &Plane) -> Channel {
-    nc::channels_fg_rgb(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_fg_rgb(plane: &NcPlane) -> Channel {
+    channels_fg_rgb(unsafe { ncplane_channels(plane) })
 }
 
 /// Extract 24 bits of working background RGB from an ncplane, shifted to LSBs.
-// TODO: TEST
 #[inline]
-pub fn ncplane_bg_rgb(plane: &Plane) -> Channel {
-    nc::channels_bg_rgb(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_bg_rgb(plane: &NcPlane) -> Channel {
+    channels_bg_rgb(unsafe { ncplane_channels(plane) })
 }
 
 /// Extract 2 bits of foreground alpha from 'struct ncplane', shifted to LSBs.
-// TODO: TEST
 #[inline]
-pub fn ncplane_fg_alpha(plane: &Plane) -> AlphaBits {
-    nc::channels_fg_alpha(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_fg_alpha(plane: &NcPlane) -> AlphaBits {
+    channels_fg_alpha(unsafe { ncplane_channels(plane) })
 }
 
 /// Extract 2 bits of background alpha from 'struct ncplane', shifted to LSBs.
-// TODO: TEST
 #[inline]
-pub fn ncplane_bg_alpha(plane: &Plane) -> AlphaBits {
-    nc::channels_bg_alpha(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_bg_alpha(plane: &NcPlane) -> AlphaBits {
+    channels_bg_alpha(unsafe { ncplane_channels(plane) })
 }
 
 /// Is the plane's foreground using the "default foreground color"?
-// TODO: TEST
 #[inline]
-pub fn ncplane_fg_default_p(plane: &Plane) -> bool {
-    nc::channels_fg_default_p(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_fg_default_p(plane: &NcPlane) -> bool {
+    channels_fg_default_p(unsafe { ncplane_channels(plane) })
 }
 
 /// Is the plane's background using the "default background color"?
-// TODO: TEST
 #[inline]
-pub fn ncplane_bg_default_p(plane: &Plane) -> bool {
-    nc::channels_bg_default_p(unsafe { nc::ncplane_channels(plane) })
+pub fn ncplane_bg_default_p(plane: &NcPlane) -> bool {
+    channels_bg_default_p(unsafe { ncplane_channels(plane) })
 }
 
 /// Extract 24 bits of foreground RGB from a plane, split into components.
-// TODO: TEST
 #[inline]
 pub fn ncplane_fg_rgb8(
-    plane: &Plane,
+    plane: &NcPlane,
     red: &mut Color,
     green: &mut Color,
     blue: &mut Color,
 ) -> Channel {
-    nc::channels_fg_rgb8(unsafe { nc::ncplane_channels(plane) }, red, green, blue)
+    channels_fg_rgb8(unsafe { ncplane_channels(plane) }, red, green, blue)
 }
 
 /// Extract 24 bits of background RGB from a plane, split into components.
-// TODO: TEST
 #[inline]
 pub fn ncplane_bg_rgb8(
-    plane: &Plane,
+    plane: &NcPlane,
     red: &mut Color,
     green: &mut Color,
     blue: &mut Color,
 ) -> Channel {
-    nc::channels_bg_rgb8(unsafe { nc::ncplane_channels(plane) }, red, green, blue)
+    channels_bg_rgb8(unsafe { ncplane_channels(plane) }, red, green, blue)
 }
 
-// TODO: TEST
+///
 #[inline]
 pub fn ncplane_rounded_box(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     stylemask: StyleMask,
-    channels: ChannelPair,
+    channels: Channels,
     ystop: i32,
     xstop: i32,
     ctlword: u32,
@@ -689,7 +674,7 @@ pub fn ncplane_rounded_box(
     let mut vl = cell_trivial_initializer![];
 
     unsafe {
-        ret = nc::cells_rounded_box(
+        ret = cells_rounded_box(
             plane,
             stylemask as u32,
             channels,
@@ -701,32 +686,32 @@ pub fn ncplane_rounded_box(
             &mut vl,
         );
         if ret == 0 {
-            ret = nc::ncplane_box(plane, &ul, &ur, &ll, &lr, &hl, &vl, ystop, xstop, ctlword);
+            ret = ncplane_box(plane, &ul, &ur, &ll, &lr, &hl, &vl, ystop, xstop, ctlword);
         }
 
-        nc::cell_release(plane, &mut ul);
-        nc::cell_release(plane, &mut ur);
-        nc::cell_release(plane, &mut ll);
-        nc::cell_release(plane, &mut lr);
-        nc::cell_release(plane, &mut hl);
-        nc::cell_release(plane, &mut vl);
+        cell_release(plane, &mut ul);
+        cell_release(plane, &mut ur);
+        cell_release(plane, &mut ll);
+        cell_release(plane, &mut lr);
+        cell_release(plane, &mut hl);
+        cell_release(plane, &mut vl);
     }
     ret
 }
 
-// TODO: TEST
+///
 #[inline]
 pub fn ncplane_rounded_box_sized(
-    plane: &mut Plane,
+    plane: &mut NcPlane,
     stylemask: StyleMask,
-    channels: ChannelPair,
+    channels: Channels,
     ylen: i32,
     xlen: i32,
     ctlword: u32,
 ) -> IntResult {
     let (mut y, mut x) = (0, 0);
     unsafe {
-        nc::ncplane_cursor_yx(plane, &mut y, &mut x);
+        ncplane_cursor_yx(plane, &mut y, &mut x);
     }
     ncplane_rounded_box(
         plane,
