@@ -40,11 +40,13 @@ typedef struct elem {
 
 static struct ncmenu* menu;
 static struct ncplane* about; // "about" modal popup
+static struct ncplane* debug; // "debug info" modal popup
 
 #define MENUSTR_TOGGLE_HUD "Toggle HUD"
 #define MENUSTR_TOGGLE_PLOT "Toggle FPS plot"
 #define MENUSTR_RESTART "Restart"
 #define MENUSTR_ABOUT "About"
+#define MENUSTR_DEBUG "Debug info"
 #define MENUSTR_QUIT "Quit"
 
 static int
@@ -60,6 +62,68 @@ hud_standard_bg_rgb(struct ncplane* n){
   return 0;
 }
 
+static int
+count_debug_lines(const char* output, size_t outputlen){
+  int lines = 0;
+  for(size_t i = 0 ; i < outputlen ; ++i){
+    if(output[i] == '\n'){
+      ++lines;
+    }
+  }
+  return lines;
+}
+
+static void
+debug_toggle(struct notcurses* nc){
+  ncmenu_rollup(menu);
+  if(debug){
+    ncplane_destroy(debug);
+    debug = NULL;
+    return;
+  }
+  int dimy, dimx;
+  notcurses_term_dim_yx(nc, &dimy, &dimx);
+  char* output = NULL;
+  size_t outputlen = 0;
+  FILE* mstream = open_memstream(&output, &outputlen);
+  if(mstream == NULL){
+    return;
+  }
+  notcurses_debug(nc, mstream);
+  if(fclose(mstream)){
+    return;
+  }
+  ncplane_options nopts = {
+    .y = 3,
+    .horiz = {
+      .align = NCALIGN_CENTER,
+    },
+    .rows = count_debug_lines(output, outputlen) + 1,
+    .cols = dimx + 1, // so we don't break lines before the true boundary
+    .flags = NCPLANE_OPTION_HORALIGNED,
+  };
+  struct ncplane* n = ncplane_create(notcurses_stdplane(nc), &nopts);
+  if(n == NULL){
+    free(output);
+    return;
+  }
+  uint64_t channels = 0;
+  channels_set_fg_alpha(&channels, CELL_ALPHA_TRANSPARENT);
+  channels_set_bg_rgb(&channels, 0xffffe5);
+  ncplane_set_base(n, " ", 0, channels);
+  ncplane_set_scrolling(n, true);
+  ncplane_set_fg_rgb(n, 0x0a0a0a);
+  ncplane_set_bg_rgb(n, 0xffffe5);
+  if(ncplane_puttext(n, 0, NCALIGN_CENTER, output, &outputlen) < 0){
+    free(output);
+    ncplane_destroy(n);
+    return;
+  }
+  ncplane_putstr_aligned(n, ncplane_dim_y(n) - 1, NCALIGN_CENTER, "Press Alt+d to hide this window");
+  free(output);
+  debug = n;
+}
+
 static void
 about_toggle(struct notcurses* nc){
   ncmenu_rollup(menu);
@@ -185,6 +249,10 @@ bool menu_or_hud_key(struct notcurses *nc, const struct ncinput *ni){
     about_toggle(nc);
     return true;
   }
+  if(tmpni.id == 'd' && tmpni.alt && !tmpni.ctrl){
+    debug_toggle(nc);
+    return true;
+  }
   if(tmpni.id == 'R' && !tmpni.alt && tmpni.ctrl){
     if(menu){
       ncmenu_rollup(menu);
@@ -204,12 +272,6 @@ bool menu_or_hud_key(struct notcurses *nc, const struct ncinput *ni){
   }
   if(ncmenu_offer_input(menu, ni)){
     return true;
-  }else if(ni->id == 'o' && ni->alt && !ni->ctrl){
-    ncmenu_unroll(menu, 0);
-    return true;
-  }else if(ni->id == 'h' && ni->alt && !ni->ctrl){
-    ncmenu_unroll(menu, 2);
-    return true;
   }
   return false;
 }
@@ -224,6 +286,7 @@ struct ncmenu* menu_create(struct notcurses* nc){
   };
   struct ncmenu_item help_items[] = {
     { .desc = MENUSTR_ABOUT, .shortcut = { .id = 'U', .ctrl = true, }, },
+    { .desc = MENUSTR_DEBUG, .shortcut = { .id = 'd', .alt = true, }, },
   };
   struct ncmenu_section sections[] = {
     { .name = "notcurses-demo", .items = demo_items,
@@ -485,6 +548,9 @@ int demo_render(struct notcurses* nc){
   if(about){
     ncplane_move_top(about);
   }
+  if(debug){
+    ncplane_move_top(debug);
+  }
   struct timespec ts;
   clock_gettime(CLOCK_MONOTONIC, &ts);
   if(plot){
