@@ -153,7 +153,7 @@ cursor_yx_get(int ttyfd, int* y, int* x){
   while(read(ttyfd, &in, 1) == 1){
     bool valid = false;
     switch(state){
-      case CURSOR_ESC: valid = (in == '\x1b'); state = CURSOR_LSQUARE; break;
+      case CURSOR_ESC: valid = (in == NCKEY_ESC); state = CURSOR_LSQUARE; break;
       case CURSOR_LSQUARE: valid = (in == '['); state = CURSOR_ROW; break;
       case CURSOR_ROW:
         if(isdigit(in)){
@@ -491,7 +491,7 @@ int ncdirect_vprintf_aligned(ncdirect* n, int y, ncalign_e align, const char* fm
   if(r == nullptr){
     return -1;
   }
-  const size_t len = strlen(r);
+  const size_t len = ncstrwidth(r);
   const int x = ncdirect_align(n, align, len);
   if(ncdirect_cursor_move_yx(n, y, x)){
     free(r);
