@@ -6,6 +6,7 @@ extern "C" {
 #endif
 
 #include "version.h"
+#include "builddef.h"
 
 #ifdef USE_FFMPEG
 #include <libavutil/error.h>
@@ -157,6 +158,7 @@ typedef struct ncmenu_int_item {
   int shortcut_offset;  // column offset with desc of shortcut EGC
   char* shortdesc;      // description of shortcut, can be NULL
   int shortdesccols;    // columns occupied by shortcut description
+  bool disabled;        // disabled?
 } ncmenu_int_item;
 
 typedef struct ncmenu_int_section {
@@ -168,6 +170,7 @@ typedef struct ncmenu_int_section {
   int bodycols;           // column width of longest item
   int itemselected;       // current item selected, -1 for no selection
   int shortcut_offset;    // column offset within name of shortcut EGC
+  int enabled_item_count; // number of enabled items: section is disabled iff 0
 } ncmenu_int_section;
 
 typedef struct ncfdplane {
@@ -208,7 +211,9 @@ typedef struct ncmenu {
   int unrolledsection;      // currently unrolled section, -1 if none
   int headerwidth;          // minimum space necessary to display all sections
   uint64_t headerchannels;  // styling for header
+  uint64_t dissectchannels; // styling for disabled section headers
   uint64_t sectionchannels; // styling for sections
+  uint64_t disablechannels; // styling for disabled entries
   bool bottom;              // are we on the bottom (vs top)?
 } ncmenu;
 
@@ -250,10 +255,10 @@ typedef struct tinfo {
   char* smkx;     // enter keypad transmit mode (keypad_xmit)
   char* rmkx;     // leave keypad transmit mode (keypad_local)
   char* getm;     // get mouse events
-  bool RGBflag;   // ti-reported "RGB" flag for 24bpc truecolor
-  bool CCCflag;   // ti-reported "CCC" flag for palette set capability
-  bool BCEflag;   // ti-reported "BCE" flag for erases with background color
-  bool AMflag;    // ti-reported "AM" flag for automatic movement to next line
+  bool RGBflag;   // "RGB" flag for 24bpc truecolor
+  bool CCCflag;   // "CCC" flag for palette set capability
+  bool BCEflag;   // "BCE" flag for erases with background color
+  bool AMflag;    // "AM" flag for automatic movement to next line
   char* smcup;    // enter alternate mode
   char* rmcup;    // restore primary mode
 } tinfo;
@@ -925,59 +930,67 @@ egc_rtl(const char* egc, int* bytes){
   return s;
 }
 
+// lowest level of cell+pool setup. if the EGC changes the output to RTL, it
+// must be suffixed with a LTR-forcing character by now, and both
+// CELL_WIDEASIAN_MASK and CELL_NOBACKGROUND_MASK ought be set however they're
+// going to be set.
 static inline int
-pool_load_direct(egcpool* pool, cell* c, const char* gcluster, int bytes, int cols){
+pool_blit_direct(egcpool* pool, cell* c, const char* gcluster, int bytes, int cols){
+  pool_release(pool, c);
   if(bytes < 0 || cols < 0){
     return -1;
   }
   if(bytes <= 1){
     assert(cols < 2);
-    pool_release(pool, c);
-    c->channels &= ~(CELL_WIDEASIAN_MASK | CELL_NOBACKGROUND_MASK);
     c->gcluster = 0;
     ((unsigned char*)&c->gcluster)[0] = *gcluster;
     return bytes;
   }
-  // FIXME also shaded blocks! ░ etc. are there combined EGCs involving these?
-  if(strcmp(gcluster, "\xe2\x96\x88")){
-    c->channels &= ~CELL_NOBACKGROUND_MASK;
-    if(cols < 2){
-      c->channels &= ~CELL_WIDEASIAN_MASK;
-    }else{
-      c->channels |= CELL_WIDEASIAN_MASK;
-    }
-  }else{
-    c->channels |= CELL_NOBACKGROUND_MASK;
-    c->channels &= ~CELL_WIDEASIAN_MASK;
-  }
-  // checks for RTL and adds U+200E if so FIXME
-  char* rtl = egc_rtl(gcluster, &bytes);
-  if(rtl){
-    gcluster = rtl;
-  }
   if(bytes <= 4){
     if(strcmp(gcluster, (const char*)&c->gcluster)){
-      pool_release(pool, c);
       c->gcluster = 0;
       memcpy(&c->gcluster, gcluster, bytes);
     }
-    if(rtl){
-      free(rtl);
-    }
     return bytes;
   }
   int eoffset = egcpool_stash(pool, gcluster, bytes);
-  if(rtl){
-    free(rtl);
-  }
   if(eoffset < 0){
     return -1;
   }
-  pool_release(pool, c);
   set_gcluster_egc(c, eoffset);
   return bytes;
 }
 
+static inline int
+pool_load_direct(egcpool* pool, cell* c, const char* gcluster, int bytes, int cols){
+  if(bytes <= 1){
+    c->channels &= ~(CELL_WIDEASIAN_MASK | CELL_NOBACKGROUND_MASK);
+  }else if(cols < 2){
+    c->channels &= ~CELL_WIDEASIAN_MASK;
+    // FIXME also shaded blocks! ░ etc. are there combined EGCs involving these?
+    if(bytes == 3 && memcmp(gcluster, "\xe2\x96\x88", 4) == 0){
+      c->channels |= CELL_NOBACKGROUND_MASK;
+    }else{
+      c->channels &= ~CELL_NOBACKGROUND_MASK;
+    }
+  }else{
+    c->channels |= CELL_WIDEASIAN_MASK;
+    c->channels &= ~CELL_NOBACKGROUND_MASK;
+  }
+  char* rtl = egc_rtl(gcluster, &bytes); // checks for RTL and adds U+200E if so
+  if(rtl){
+    gcluster = rtl;
+  }
+  int r = pool_blit_direct(pool, c, gcluster, bytes, cols);
+  free(rtl);
+  return r;
+}
+
+static inline int
+cell_load_direct(ncplane* n, cell* c, const char* gcluster, int bytes, int cols){
+  return pool_load_direct(&n->pool, c, gcluster, bytes, cols);
+}
+
 static inline int
 pool_load(egcpool* pool, cell* c, const char* gcluster){
   int cols;
