@@ -22,6 +22,7 @@ static int
 dup_menu_item(ncmenu_int_item* dst, const struct ncmenu_item* src){
 #define ALTMOD "Alt+"
 #define CTLMOD "Ctrl+"
+  dst->disabled = false;
   if((dst->desc = strdup(src->desc)) == NULL){
     return -1;
   }
@@ -80,6 +81,7 @@ dup_menu_section(ncmenu_int_section* dst, const struct ncmenu_section* src){
   // we must reject any section which is entirely separators
   bool gotitem = false;
   dst->itemcount = 0;
+  dst->enabled_item_count = 0;
   dst->items = malloc(sizeof(*dst->items) * src->itemcount);
   if(dst->items == NULL){
     return -1;
@@ -113,6 +115,7 @@ dup_menu_section(ncmenu_int_section* dst, const struct ncmenu_section* src){
     }
     ++dst->itemcount;
   }
+  dst->enabled_item_count = dst->itemcount;
   if(!gotitem){
     while(dst->itemcount){
       free(dst->items[--dst->itemcount].desc);
@@ -228,7 +231,8 @@ section_x(const ncmenu* ncm, int x){
 }
 
 static int
-write_header(ncmenu* ncm){ ncm->ncp->channels = ncm->headerchannels;
+write_header(ncmenu* ncm){
+  ncplane_set_channels(ncm->ncp, ncm->headerchannels);
   int dimy, dimx;
   ncplane_dim_yx(ncm->ncp, &dimy, &dimx);
   int xoff = 0; // 2-column margin on left
@@ -260,6 +264,11 @@ write_header(ncmenu* ncm){ ncm->ncp->channels = ncm->headerchannels;
           return -1;
         }
       }
+      if(ncm->sections[i].enabled_item_count <= 0){
+        ncplane_set_channels(ncm->ncp, ncm->dissectchannels);
+      }else{
+        ncplane_set_channels(ncm->ncp, ncm->headerchannels);
+      }
       if(ncplane_putstr_yx(ncm->ncp, ypos, xoff, ncm->sections[i].name) < 0){
         return -1;
       }
@@ -292,6 +301,7 @@ ncmenu* ncmenu_create(ncplane* n, const ncmenu_options* opts){
     opts = &zeroed;
   }
   if(opts->sectioncount <= 0 || !opts->sections){
+    logerror(n->nc, "Invalid %d-ary section information\n", opts->sectioncount);
     return NULL;
   }
   if(opts->flags > NCMENU_OPTION_HIDING){
@@ -316,7 +326,11 @@ ncmenu* ncmenu_create(ncplane* n, const ncmenu_options* opts){
       if(ret->ncp){
         ret->unrolledsection = -1;
         ret->headerchannels = opts->headerchannels;
+        ret->dissectchannels = opts->headerchannels;
+        channels_set_fg_rgb(&ret->dissectchannels, 0xdddddd);
         ret->sectionchannels = opts->sectionchannels;
+        ret->disablechannels = ret->sectionchannels;
+        channels_set_fg_rgb(&ret->disablechannels, 0xdddddd);
         cell c = CELL_TRIVIAL_INITIALIZER;
         cell_set_fg_alpha(&c, CELL_ALPHA_TRANSPARENT);
         cell_set_bg_alpha(&c, CELL_ALPHA_TRANSPARENT);
@@ -345,12 +359,16 @@ section_width(const ncmenu* n, int sectionidx){
 }
 
 int ncmenu_unroll(ncmenu* n, int sectionidx){
-  if(sectionidx < 0 || sectionidx >= n->sectioncount){
-    return -1;
-  }
   if(ncmenu_rollup(n)){ // roll up any unrolled section
     return -1;
   }
+  if(sectionidx < 0 || sectionidx >= n->sectioncount){
+    logerror(n->ncp->nc, "Unrolled invalid sectionidx %d\n", sectionidx);
+    return -1;
+  }
+  if(n->sections[sectionidx].enabled_item_count <= 0){
+    return 0;
+  }
   if(n->sections[sectionidx].name == NULL){
     return -1;
   }
@@ -375,7 +393,12 @@ int ncmenu_unroll(ncmenu* n, int sectionidx){
   for(int i = 0 ; i < sec->itemcount ; ++i){
     ++ypos;
     if(sec->items[i].desc){
-      n->ncp->channels = n->sectionchannels;
+      // FIXME the user ought be able to configure the disabled channel
+      if(!sec->items[i].disabled){
+        ncplane_set_channels(n->ncp, n->sectionchannels);
+      }else{
+        ncplane_set_channels(n->ncp, n->disablechannels);
+      }
       if(i == sec->itemselected){
         ncplane_set_styles(n->ncp, NCSTYLE_REVERSE);
       }else{
@@ -443,36 +466,26 @@ int ncmenu_rollup(ncmenu* n){
 }
 
 int ncmenu_nextsection(ncmenu* n){
-  int nextsection = n->unrolledsection + 1;
-  if(nextsection){
-    ncmenu_rollup(n);
-    if(nextsection == n->sectioncount){
-      nextsection = 0;
-    }
-  }
-  if(n->sections[nextsection].name == NULL){
+  int nextsection = n->unrolledsection;
+  // FIXME probably best to detect cycles
+  do{
     if(++nextsection == n->sectioncount){
       nextsection = 0;
     }
-  }
+  }while(n->sections[nextsection].name == NULL ||
+         n->sections[nextsection].enabled_item_count == 0);
   return ncmenu_unroll(n, nextsection);
 }
 
 int ncmenu_prevsection(ncmenu* n){
   int prevsection = n->unrolledsection;
-  if(n->unrolledsection < 0){
-    prevsection = 1;
-  }else{
-    ncmenu_rollup(n);
-  }
-  if(--prevsection == -1){
-    prevsection = n->sectioncount - 1;
-  }
-  if(n->sections[prevsection].name == NULL){
+  // FIXME probably best to detect cycles
+  do{
     if(--prevsection < 0){
       prevsection = n->sectioncount - 1;
     }
-  }
+  }while(n->sections[prevsection].name == NULL ||
+         n->sections[prevsection].enabled_item_count == 0);
   return ncmenu_unroll(n, prevsection);
 }
 
@@ -482,11 +495,13 @@ int ncmenu_nextitem(ncmenu* n){
       return -1;
     }
   }
+  ncmenu_int_section* sec = &n->sections[n->unrolledsection];
+  // FIXME probably best to detect cycles
   do{
-    if(++n->sections[n->unrolledsection].itemselected == n->sections[n->unrolledsection].itemcount){
-      n->sections[n->unrolledsection].itemselected = 0;
+    if(++sec->itemselected == sec->itemcount){
+      sec->itemselected = 0;
     }
-  }while(!n->sections[n->unrolledsection].items[n->sections[n->unrolledsection].itemselected].desc);
+  }while(!sec->items[sec->itemselected].desc || sec->items[sec->itemselected].disabled);
   return ncmenu_unroll(n, n->unrolledsection);
 }
 
@@ -496,11 +511,13 @@ int ncmenu_previtem(ncmenu* n){
       return -1;
     }
   }
+  ncmenu_int_section* sec = &n->sections[n->unrolledsection];
+  // FIXME probably best to detect cycles
   do{
-    if(n->sections[n->unrolledsection].itemselected-- == 0){
-      n->sections[n->unrolledsection].itemselected = n->sections[n->unrolledsection].itemcount - 1;
+    if(sec->itemselected-- == 0){
+      sec->itemselected = sec->itemcount - 1;
     }
-  }while(!n->sections[n->unrolledsection].items[n->sections[n->unrolledsection].itemselected].desc);
+  }while(!sec->items[sec->itemselected].desc || sec->items[sec->itemselected].disabled);
   return ncmenu_unroll(n, n->unrolledsection);
 }
 
@@ -529,7 +546,7 @@ const char* ncmenu_mouse_selected(const ncmenu* n, const ncinput* click,
   if(!ncplane_translate_abs(nc, &y, &x)){
     return NULL;
   }
-  // FIXME section_x() works only off the section header lenghts, meaning that
+  // FIXME section_x() works only off the section header lengths, meaning that
   // if we click an item outside of those columns covered by the header, it will
   // read as a -1 from section_x(). we want to instead get the unrolled section,
   // find its boundaries, and verify that we are within them.
@@ -569,7 +586,19 @@ bool ncmenu_offer_input(ncmenu* n, const ncinput* nc){
       ncmenu_unroll(n, i);
     }
     return true;
-  }else if(n->unrolledsection < 0){ // all following need an unrolled section
+  }
+  for(int si = 0 ; si < n->sectioncount ; ++si){
+    const ncmenu_int_section* sec = &n->sections[si];
+    if(sec->enabled_item_count == 0){
+      continue;
+    }
+    if(!ncinput_equal_p(&sec->shortcut, nc)){
+      continue;
+    }
+    ncmenu_unroll(n, si);
+    return true;
+  }
+  if(n->unrolledsection < 0){ // all following need an unrolled section
     return false;
   }
   if(nc->id == NCKEY_LEFT){
@@ -592,13 +621,51 @@ bool ncmenu_offer_input(ncmenu* n, const ncinput* nc){
       return false;
     }
     return true;
-  }else if(nc->id == 0x1b){
+  }else if(nc->id == NCKEY_ESC){
     ncmenu_rollup(n);
     return true;
   }
   return false;
 }
 
+// FIXME we probably ought implement this with a trie or something
+int ncmenu_item_set_status(ncmenu* n, const char* section, const char* item,
+                           bool enabled){
+  for(int si = 0 ; si < n->sectioncount ; ++si){
+    struct ncmenu_int_section* sec = &n->sections[si];
+    if(strcmp(sec->name, section) == 0){
+      for(int ii = 0 ; ii < sec->itemcount ; ++ii){
+        struct ncmenu_int_item* i = &sec->items[ii];
+        if(strcmp(i->desc, item) == 0){
+          const bool changed = i->disabled == enabled;
+          i->disabled = !enabled;
+          if(changed){
+            if(i->disabled){
+              if(--sec->enabled_item_count == 0){
+                write_header(n);
+              }
+            }else{
+              if(++sec->enabled_item_count == 1){
+                write_header(n);
+              }
+            }
+            if(n->unrolledsection == si){
+              if(sec->enabled_item_count == 0){
+                ncmenu_rollup(n);
+              }else{
+                ncmenu_unroll(n, n->unrolledsection);
+              }
+            }
+          }
+          return 0;
+        }
+      }
+      break;
+    }
+  }
+  return -1;
+}
+
 ncplane* ncmenu_plane(ncmenu* menu){
   return menu->ncp;
 }
