@@ -122,7 +122,7 @@ int ncreader_move_right(ncreader* n){
 //fprintf(stderr, "moving right: tcurs: %dx%d vcurs: %dx%d xproj: %d\n", y, textx, y, viewx, n->xproject);
   if(textx >= n->textarea->lenx - 1){
     // are we on the last column of the textarea? if so, we must also be on
-    // the first column of the viewarea. try to move up.
+    // the first column of the viewarea. try to move down.
     if(y >= n->textarea->leny - 1){
       return -1; // no move possible
     }
@@ -214,19 +214,87 @@ int ncreader_write_egc(ncreader* n, const char* egc){
   return 0;
 }
 
+static bool
+do_backspace(ncreader* n){
+  int x = n->textarea->x;
+  int y = n->textarea->y;
+  if(n->textarea->x == 0){
+    if(n->textarea->y){
+      y = n->textarea->y - 1;
+      x = n->textarea->lenx - 1;
+    }
+  }else{
+    --x;
+  }
+  ncplane_putegc_yx(n->textarea, y, x, "", NULL);
+  ncplane_cursor_move_yx(n->textarea, y, x);
+  ncplane_cursor_move_yx(n->ncp, n->ncp->y, n->ncp->x - 1);
+  ncreader_redraw(n);
+  return true;
+}
+
+static bool
+is_egc_wordbreak(ncplane* textarea){
+  char* egc = ncplane_at_yx(textarea, textarea->y, textarea->x, NULL, NULL);
+fprintf(stderr, "TAKING A LOKO AT [%s]\n", egc);
+  if(egc == NULL){
+    return true;
+  }
+  wchar_t w;
+  mbstate_t mbstate = {0};
+  size_t s = mbrtowc(&w, egc, MB_CUR_MAX, &mbstate);
+  free(egc);
+  if(s == (size_t)-1 || s == (size_t)-2){
+    return true;
+  }
+  if(iswordbreak(w)){
+    return true;
+  }
+  return false;
+}
+
 static bool
 ncreader_ctrl_input(ncreader* n, const ncinput* ni){
   switch(ni->id){
-    case 'U':
-      ncplane_erase(n->ncp); // homes the cursor
-      ncplane_erase(n->textarea);
-      break;
     case 'B':
       ncreader_move_left(n);
       break;
     case 'F':
       ncreader_move_right(n);
       break;
+    case 'A': // cursor to beginning of line
+      while(n->textarea->x){
+        if(ncreader_move_left(n)){
+          break;
+        }
+      }
+      break;
+    case 'E': // cursor to end of line
+      while(n->textarea->x < ncplane_dim_x(n->textarea) - 1){
+        if(ncreader_move_right(n)){
+          break;
+        }
+      }
+      break;
+    case 'U': // clear line before cursor
+      while(n->textarea->x){
+        do_backspace(n);
+      }
+      break;
+    case 'W': // clear word before cursor
+      while(n->textarea->x){
+        if(ncreader_move_left(n)){
+          break;
+        }
+        if(is_egc_wordbreak(n->textarea)){
+          break;
+        }
+        if(ncreader_move_right(n)){
+          break;
+        }
+        do_backspace(n);
+      }
+      break;
     default:
       return false; // pass on all other ctrls
   }
@@ -236,11 +304,25 @@ ncreader_ctrl_input(ncreader* n, const ncinput* ni){
 static bool
 ncreader_alt_input(ncreader* n, const ncinput* ni){
   switch(ni->id){
-    case 'B': // back one word (to first cell) FIXME
-      ncreader_move_left(n);
+    case 'b': // back one word (to first cell), but not to previous line
+      while(n->textarea->x){
+        if(ncreader_move_left(n)){
+          break;
+        }
+        if(is_egc_wordbreak(n->textarea)){
+          break;
+        }
+      }
       break;
-    case 'F': // forward one word (past end cell) FIXME
-      ncreader_move_right(n);
+    case 'f': // forward one word (past end cell)
+      while(n->textarea->x < ncplane_dim_x(n->textarea) - 1){
+        if(ncreader_move_right(n)){
+          break;
+        }
+        if(is_egc_wordbreak(n->textarea)){
+          break;
+        }
+      }
       break;
     default:
       return false;
@@ -253,30 +335,16 @@ ncreader_alt_input(ncreader* n, const ncinput* ni){
 //  * anything with Ctrl, except 'U' (which clears all input)
 //  * anything synthesized, save arrow keys and backspace
 bool ncreader_offer_input(ncreader* n, const ncinput* ni){
-  int x = n->textarea->x;
-  int y = n->textarea->y;
-  if(ni->alt){ // pass on all alts
-    return false;
-  }
   if(ni->ctrl && !n->no_cmd_keys){
     return ncreader_ctrl_input(n, ni);
   }else if(ni->alt && !n->no_cmd_keys){
     return ncreader_alt_input(n, ni);
   }
+  if(ni->alt || ni->ctrl){ // pass on all alts/ctrls if no_cmd_keys is set
+    return false;
+  }
   if(ni->id == NCKEY_BACKSPACE){
-    if(n->textarea->x == 0){
-      if(n->textarea->y){
-        y = n->textarea->y - 1;
-        x = n->textarea->lenx - 1;
-      }
-    }else{
-      --x;
-    }
-    ncplane_putegc_yx(n->textarea, y, x, "", NULL);
-    ncplane_cursor_move_yx(n->textarea, y, x);
-    ncplane_cursor_move_yx(n->ncp, n->ncp->y, n->ncp->x - 1);
-    ncreader_redraw(n);
-    return true;
+    return do_backspace(n);
   }
   // FIXME deal with multicolumn EGCs -- probably extract these and make them
   // general ncplane_cursor_{left, right, up, down}()
