@@ -1,6 +1,6 @@
 #include <cmath>
 #include <cstring>
-#include "version.h"
+#include "builddef.h"
 #include "visual-details.h"
 #include "internal.h"
 
@@ -59,14 +59,12 @@ auto bgra_to_rgba(const void* data, int rows, int rowstride, int cols) -> void*
   if(rowstride % 4){ // must be a multiple of 4 bytes
     return nullptr;
   }
-//fprintf(stderr, "ROWS: %d\n", rows);
   auto ret = static_cast<uint32_t*>(malloc(rowstride * rows));
   if(ret){
     for(int y = 0 ; y < rows ; ++y){
       for(int x = 0 ; x < cols ; ++x){
         const uint32_t* src = (const uint32_t*)data + (rowstride / 4) * y + x;
         uint32_t* dst = ret + (rowstride / 4) * y + x;
-//fprintf(stderr, "SRC: %x DST: %x\n", *src, *dst);
         *dst = ((*src & 0xff00u) << 16u) | (*src & 0xff00ffu) | ((*src & 0xff000000) >> 16u);
 //fprintf(stderr, "y: %d x: %d SRC: %x DST: %x\n", y, x, *src, *dst);
       }
@@ -346,6 +344,11 @@ auto ncvisual_from_bgra(const void* bgra, int rows, int rowstride,
     ncv->cols = cols;
     ncv->rows = rows;
     auto data = static_cast<uint32_t*>(memdup(bgra, rowstride * ncv->rows));
+    for(int p = 0 ; p < rowstride / 4 * ncv->rows ; ++p){
+      const unsigned r = (data[p] & 0xffllu) << 16u;
+      const unsigned b = (data[p] & 0xff0000llu) >> 16u;
+      data[p] = (data[p] & 0xff00ff00llu) | r | b;
+    }
     if(data == nullptr){
       ncvisual_destroy(ncv);
       return nullptr;
@@ -574,6 +577,11 @@ auto ncvisual_decode(ncvisual* nc) -> int {
   return -1;
 }
 
+auto ncvisual_decode_loop(ncvisual* nc) -> int {
+  (void)nc;
+  return -1;
+}
+
 auto ncvisual_stream(notcurses* nc, ncvisual* ncv, float timescale,
                      streamcb streamer, const ncvisual_options* vopts,
                      void* curry) -> int {
@@ -603,7 +611,7 @@ auto ncvisual_blit(ncvisual* ncv, int rows, int cols, ncplane* n,
   (void)rows;
   (void)cols;
   if(rgba_blit_dispatch(n, bset, placey, placex, ncv->rowstride, ncv->data,
-                        begy, begx, leny, lenx, blendcolors) <= 0){
+                        begy, begx, leny, lenx, blendcolors) < 0){
     return -1;
   }
   return 0;
