@@ -17,10 +17,14 @@ static void usage(std::ostream& os, const char* name, int exitcode)
   __attribute__ ((noreturn));
 
 void usage(std::ostream& o, const char* name, int exitcode){
-  o << "usage: " << name << " [ -h ] [ -m margins ] [ -l loglevel ] [ -d mult ] [ -s scaletype ] [ -k ] files" << '\n';
+  o << "usage: " << name << " [ -h ] [ -q ] [ -m margins ] [ -l loglevel ] [ -d mult ] [ -s scaletype ] [ -k ] [ -L ] files" << '\n';
+  o << " -h: display help and exit with success\n";
+  o << " -q: be quiet (no frame/timing information along top of screen)\n";
   o << " -k: don't use the alternate screen\n";
+  o << " -L: loop frames\n";
   o << " -l loglevel: integer between 0 and 9, goes to stderr'\n";
   o << " -s scaletype: one of 'none', 'scale', or 'stretch'\n";
+  o << " -b blitter: 'ascii', 'halfblock', 'quadblitter', 'sexblitter', or 'braille'\n";
   o << " -m margins: margin, or 4 comma-separated margins\n";
   o << " -d mult: non-negative floating point scale for frame time" << std::endl;
   exit(exitcode);
@@ -40,12 +44,17 @@ ns_to_timespec(uint64_t ns, struct timespec* ts){
   return ts;
 }
 
-// FIXME internalize this via complex curry
-static struct ncplane* subtitle_plane = nullptr;
+struct marshal {
+  struct ncplane* subtitle_plane;
+  int framecount;
+  bool quiet;
+  ncblitter_e blitter; // can be changed while streaming, must propagate out
+};
 
 // frame count is in the curry. original time is kept in n's userptr.
 auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
-              const struct timespec* abstime, void* vframecount) -> int {
+              const struct timespec* abstime, void* vmarshal) -> int {
+  struct marshal* marsh = static_cast<struct marshal*>(vmarshal);
   NotCurses &nc = NotCurses::get_instance ();
   auto start = static_cast<struct timespec*>(ncplane_userptr(vopts->n));
   if(!start){
@@ -54,38 +63,40 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
     ncplane_set_userptr(vopts->n, start);
   }
   std::unique_ptr<Plane> stdn(nc.get_stdplane());
-  int* framecount = static_cast<int*>(vframecount);
-  ++*framecount;
+  // negative framecount means don't print framecount/timing (quiet mode)
+  if(marsh->framecount >= 0){
+    ++marsh->framecount;
+  }
   stdn->set_fg_rgb(0x80c080);
   struct timespec now;
   clock_gettime(CLOCK_MONOTONIC, &now);
   intmax_t ns = timespec_to_ns(&now) - timespec_to_ns(start);
-  auto blitter = vopts->blitter;
-  if(blitter == NCBLIT_DEFAULT){
-    blitter = ncvisual_default_blitter(notcurses_canutf8(nc), vopts->scaling);
-    vopts->blitter = blitter;
+  marsh->blitter = vopts->blitter;
+  if(marsh->blitter == NCBLIT_DEFAULT){
+    marsh->blitter = ncvisual_default_blitter(notcurses_canutf8(nc), vopts->scaling);
+  }
+  if(!marsh->quiet){
+    stdn->printf(0, NCAlign::Left, "frame %06d\u2026 (%s)", marsh->framecount,
+                 notcurses_str_blitter(marsh->blitter));
   }
-  // clear top line only
-  stdn->printf(0, NCAlign::Left, "frame %06d\u2026 (%s)", *framecount,
-               notcurses_str_blitter(blitter));
   char* subtitle = ncvisual_subtitle(ncv);
   if(subtitle){
-    if(!subtitle_plane){
+    if(!marsh->subtitle_plane){
       int dimx, dimy;
       ncplane_dim_yx(vopts->n, &dimy, &dimx);
-      subtitle_plane = ncplane_new(notcurses_stdplane(nc), 1, dimx,
-                                   dimy - 1, 0, nullptr, "subt");
+      marsh->subtitle_plane = ncplane_new(notcurses_stdplane(nc), 1, dimx,
+                                          dimy - 1, 0, nullptr, "subt");
       uint64_t channels = 0;
       channels_set_fg_alpha(&channels, CELL_ALPHA_TRANSPARENT);
       channels_set_bg_alpha(&channels, CELL_ALPHA_TRANSPARENT);
-      ncplane_set_base(subtitle_plane, "", 0, channels);
-      ncplane_set_fg_rgb(subtitle_plane, 0x00ffff);
-      ncplane_set_fg_alpha(subtitle_plane, CELL_ALPHA_HIGHCONTRAST);
-      ncplane_set_bg_alpha(subtitle_plane, CELL_ALPHA_TRANSPARENT);
+      ncplane_set_base(marsh->subtitle_plane, "", 0, channels);
+      ncplane_set_fg_rgb(marsh->subtitle_plane, 0x00ffff);
+      ncplane_set_fg_alpha(marsh->subtitle_plane, CELL_ALPHA_HIGHCONTRAST);
+      ncplane_set_bg_alpha(marsh->subtitle_plane, CELL_ALPHA_TRANSPARENT);
     }else{
-      ncplane_erase(subtitle_plane);
+      ncplane_erase(marsh->subtitle_plane);
     }
-    ncplane_printf_yx(subtitle_plane, 0, 0, "%s", subtitle);
+    ncplane_printf_yx(marsh->subtitle_plane, 0, 0, "%s", subtitle);
     free(subtitle);
   }
   const intmax_t h = ns / (60 * 60 * NANOSECS_IN_SEC);
@@ -94,8 +105,10 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
   ns -= m * (60 * NANOSECS_IN_SEC);
   const intmax_t s = ns / NANOSECS_IN_SEC;
   ns -= s * NANOSECS_IN_SEC;
-  stdn->printf(0, NCAlign::Right, "%02jd:%02jd:%02jd.%04jd",
-               h, m, s, ns / 1000000);
+  if(!marsh->quiet){
+    stdn->printf(0, NCAlign::Right, "%02jd:%02jd:%02jd.%04jd",
+                 h, m, s, ns / 1000000);
+  }
   if(!nc.render()){
     return -1;
   }
@@ -132,7 +145,20 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
     if(keyp == NCKey::Resize){
       return 0;
     }else if(keyp >= '0' && keyp <= '8'){ // FIXME eliminate ctrl/alt
-      vopts->blitter = static_cast<ncblitter_e>(keyp - '0');
+      marsh->blitter = static_cast<ncblitter_e>(keyp - '0');
+      vopts->blitter = marsh->blitter;
+      continue;
+    }else if(keyp == NCKey::Up){
+      // FIXME
+      continue;
+    }else if(keyp == NCKey::Down){
+      // FIXME
+      continue;
+    }else if(keyp == NCKey::Right){
+      // FIXME
+      continue;
+    }else if(keyp == NCKey::Left){
+      // FIXME
       continue;
     }
     return 1;
@@ -141,16 +167,21 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
 }
 
 // can exit() directly. returns index in argv of first non-option param.
-auto handle_opts(int argc, char** argv, notcurses_options& opts,
-                 float* timescale, ncscale_e* scalemode) -> int {
+auto handle_opts(int argc, char** argv, notcurses_options& opts, bool* quiet,
+                 float* timescale, ncscale_e* scalemode, ncblitter_e* blitter,
+                 bool* loop)
+                 -> int {
   *timescale = 1.0;
   *scalemode = NCSCALE_STRETCH;
   int c;
-  while((c = getopt(argc, argv, "hl:d:s:m:k")) != -1){
+  while((c = getopt(argc, argv, "hql:d:s:b:m:kL")) != -1){
     switch(c){
       case 'h':
         usage(std::cout, argv[0], EXIT_SUCCESS);
         break;
+      case 'q':
+        *quiet = true;
+        break;
       case 's':
         if(notcurses_lex_scalemode(optarg, scalemode)){
           std::cerr << "Scaling type should be one of stretch, scale, none (got "
@@ -158,9 +189,19 @@ auto handle_opts(int argc, char** argv, notcurses_options& opts,
           usage(std::cerr, argv[0], EXIT_FAILURE);
         }
         break;
+      case 'b':
+        if(notcurses_lex_blitter(optarg, blitter)){
+          std::cerr << "Invalid blitter specification (got "
+                    << optarg << ")" << std::endl;
+          usage(std::cerr, argv[0], EXIT_FAILURE);
+        }
+        break;
       case 'k':{
         opts.flags |= NCOPTION_NO_ALTERNATE_SCREEN;
         break;
+      }case 'L':{
+        *loop = true;
+        break;
       }case 'm':{
         if(opts.margin_t || opts.margin_r || opts.margin_b || opts.margin_l){
           std::cerr <<  "Provided margins twice!" << std::endl;
@@ -216,7 +257,11 @@ auto main(int argc, char** argv) -> int {
   float timescale;
   ncscale_e scalemode;
   notcurses_options nopts{};
-  auto nonopt = handle_opts(argc, argv, nopts, &timescale, &scalemode);
+  ncblitter_e blitter = NCBLIT_DEFAULT;
+  bool quiet = false;
+  bool loop = false;
+  auto nonopt = handle_opts(argc, argv, nopts, &quiet, &timescale, &scalemode,
+                            &blitter, &loop);
   nopts.flags |= NCOPTION_INHIBIT_SETLOCALE;
   NotCurses nc{nopts};
   if(!nc.can_open_images()){
@@ -224,13 +269,11 @@ auto main(int argc, char** argv) -> int {
     std::cerr << "Notcurses was compiled without multimedia support\n";
     return EXIT_FAILURE;
   }
-  ncblitter_e blitter = NCBLIT_DEFAULT;
   int dimy, dimx;
   bool failed = false;
   {
     std::unique_ptr<Plane> stdn(nc.get_stdplane(&dimy, &dimx));
     for(auto i = nonopt ; i < argc ; ++i){
-      int frames = 0;
       std::unique_ptr<Visual> ncv;
       try{
         ncv = std::make_unique<Visual>(argv[i]);
@@ -244,38 +287,53 @@ auto main(int argc, char** argv) -> int {
         stdn->erase();
       }
       struct ncvisual_options vopts{};
+      int r;
       vopts.n = *stdn;
       vopts.scaling = scalemode;
       vopts.blitter = blitter;
-      int r = ncv->stream(&vopts, timescale, perframe, &frames);
-      free(stdn->get_userptr());
-      stdn->set_userptr(nullptr);
+      do{
+        struct marshal marsh = {
+          .subtitle_plane = nullptr,
+          .framecount = 0,
+          .quiet = quiet,
+          .blitter = vopts.blitter,
+        };
+        r = ncv->stream(&vopts, timescale, perframe, &marsh);
+        free(stdn->get_userptr());
+        stdn->set_userptr(nullptr);
+        if(r == 0){
+          vopts.blitter = marsh.blitter;
+          if(!loop){
+            stdn->printf(0, NCAlign::Center, "press any key to advance");
+            if(!nc.render()){
+              failed = true;
+              break;
+            }
+            char32_t ie = nc.getc(true);
+            if(ie == (char32_t)-1){
+              failed = true;
+              break;
+            }else if(ie == 'q'){
+              break;
+            }else if(ie >= '0' && ie <= '8'){
+              --i; // rerun same input with the new blitter
+              vopts.blitter = static_cast<ncblitter_e>(ie - '0');
+            }else if(ie == NCKey::Resize){
+              --i; // rerun with the new size
+              if(!nc.refresh(&dimy, &dimx)){
+                failed = true;
+                break;
+              }
+            }
+          }else{
+            ncv->decode_loop();
+          }
+        }
+      }while(loop && r == 0);
       if(r < 0){ // positive is intentional abort
         std::cerr << "Error decoding " << argv[i] << std::endl;
         failed = true;
         break;
-      }else if(r == 0){
-        stdn->printf(0, NCAlign::Center, "press any key to advance");
-        if(!nc.render()){
-          failed = true;
-          break;
-        }
-        char32_t ie = nc.getc(true);
-        if(ie == (char32_t)-1){
-          failed = true;
-          break;
-        }else if(ie == 'q'){
-          break;
-        }else if(ie >= '0' && ie <= '8'){
-          --i; // rerun same input with the new blitter
-          blitter = static_cast<ncblitter_e>(ie - '0');
-        }else if(ie == NCKey::Resize){
-          --i; // rerun with the new size
-          if(!nc.refresh(&dimy, &dimx)){
-            failed = true;
-            break;
-          }
-        }
       }
     }
   }
